from django.contrib.auth.hashers import make_password
from django.forms import PasswordInput
from django.utils.translation import gettext_lazy as _

from dynamic_preferences.preferences import Section
from dynamic_preferences.serializers import StringSerializer
from dynamic_preferences.types import (
    BooleanPreference,
    StringPreference,
)

from aleksis.core.registries import site_preferences_registry

maka = Section("maka", verbose_name=_("Grade management"))


class PasswordSerializer(StringSerializer):
    """Serializer hashing the given string before saving it."""

    @classmethod
    def to_db(cls, value, **kwargs):
        return str(make_password(value))


@site_preferences_registry.register
class SharedSecret(StringPreference):
    """Shared secret for accessing grade management pages."""

    section = maka
    name = "shared_secret"
    default = make_password("aleksis")
    required = True
    widget = PasswordInput
    serializer = PasswordSerializer
    verbose_name = _("Shared secret")
    help_text = _(
        "This is the global shared secret key used to access all grade "
        "management pages requiring two factor authentication. It is not possible "
        "to see the secret again after setting it; please write it down safely."
    )


@site_preferences_registry.register
class ViewOwnEfforts(BooleanPreference):
    """Preference whether persons should be allowed to view the efforts linked to groups they're members in."""

    section = maka
    name = "view_own_efforts"
    default = False
    verbose_name = _("Allow viewing own efforts")


@site_preferences_registry.register
class ViewOwnGrades(BooleanPreference):
    """Preference whether persons should be allowed to view the grades given to them."""

    section = maka
    name = "view_own_grades"
    default = False
    verbose_name = _("Allow viewing own grades")
