from rules import add_perm

from aleksis.core.models import Group
from aleksis.core.util.predicates import (
    has_any_object,
    has_global_perm,
    has_object_perm,
    has_person,
    is_current_person,
    is_site_preference_set,
)

from .models import EffortType, GradeSet
from .util.predicates import (
    has_any_group_ownership,
    has_effort_group_perm,
    has_grade_group_perm,
    is_effort_group_member,
    is_effort_group_owner,
    is_grade_group_owner,
    is_grade_person,
)

# Efforts
view_effort_predicate = has_person & (
    (is_effort_group_member & is_site_preference_set("maka", "view_own_efforts"))
    | is_effort_group_owner
    | has_global_perm("maka.view_effort")
    | has_effort_group_perm("core.view_efforts_group")
)
add_perm("maka.view_effort_rule", view_effort_predicate)

add_effort_predicate = has_person & (
    is_effort_group_owner
    | has_global_perm("maka.add_effort")
    | has_effort_group_perm("core.manage_efforts_group")
)
add_perm("maka.add_effort_rule", add_effort_predicate)

edit_effort_predicate = has_person & (
    is_effort_group_owner
    | has_global_perm("maka.change_effort")
    | has_effort_group_perm("core.manage_efforts_group")
)
add_perm("maka.edit_effort_rule", edit_effort_predicate)

delete_effort_predicate = has_person & (
    is_effort_group_owner
    | has_global_perm("maka.delete_effort")
    | has_effort_group_perm("core.manage_efforts_group")
)
add_perm("maka.delete_effort_rule", delete_effort_predicate)

view_efforts_menu_predicate = has_person & (
    has_global_perm("maka.view_effort")
    | has_any_group_ownership
    | has_any_object("core.view_efforts_group", Group)
)
add_perm("maka.view_efforts_menu_rule", view_efforts_menu_predicate)

# Grades

view_grade_predicate = has_person & (
    (is_grade_person & is_site_preference_set("maka", "view_own_grades"))
    | is_grade_group_owner
    | has_global_perm("maka.view_grade")
    | has_grade_group_perm("core.view_grades_group")
)
add_perm("maka.view_grade_rule", view_grade_predicate)

add_grade_predicate = has_person & (
    is_grade_group_owner
    | has_global_perm("maka.add_grade")
    | has_grade_group_perm("core.manage_grades_group")
)
add_perm("maka.add_grade_rule", add_grade_predicate)

edit_grade_predicate = has_person & (
    is_grade_group_owner
    | has_global_perm("maka.change_grade")
    | has_grade_group_perm("core.manage_grades_group")
)
add_perm("maka.edit_grade_rule", edit_grade_predicate)

delete_grade_predicate = has_person & (
    is_grade_group_owner
    | has_global_perm("maka.delete_grade")
    | has_grade_group_perm("core.manage_grades_group")
)
add_perm("maka.delete_grade_rule", delete_grade_predicate)

# Grade sets

view_gradesets_predicate = has_person
add_perm("maka.view_gradesets_rule", view_gradesets_predicate)

add_gradeset_predicate = view_gradesets_predicate & has_global_perm("maka.add_gradeset")
add_perm("maka.add_gradeset_rule", add_gradeset_predicate)

edit_gradeset_predicate = view_gradesets_predicate & has_global_perm("maka.change_gradeset")
add_perm("maka.edit_gradeset_rule", edit_gradeset_predicate)

delete_gradeset_predicate = view_gradesets_predicate & has_global_perm("maka.delete_gradeset")
add_perm("maka.delete_gradeset_rule", delete_gradeset_predicate)

view_gradesets_menu_predicate = has_person & (
    has_global_perm("maka.view_gradeset")
    | has_any_object("maka.view_gradeset", GradeSet)
)
add_perm("maka.view_gradesets_menu_rule", view_gradesets_menu_predicate)

# Grade choices

view_gradechoices_predicate = has_person
add_perm("maka.view_gradechoices_rule", view_gradechoices_predicate)

add_gradechoice_predicate = view_gradechoices_predicate & has_global_perm("maka.add_gradechoice")
add_perm("maka.add_gradechoice_rule", add_gradechoice_predicate)

edit_gradechoice_predicate = view_gradechoices_predicate & has_global_perm("maka.change_gradechoice")
add_perm("maka.edit_gradechoice_rule", edit_gradechoice_predicate)

delete_gradechoice_predicate = view_gradechoices_predicate & has_global_perm("maka.delete_gradechoice")
add_perm("maka.delete_gradechoice_rule", delete_gradechoice_predicate)

# Effort types

view_efforttypes_predicate = has_person
add_perm("maka.view_efforttypes_rule", view_efforttypes_predicate)

add_efforttype_predicate = view_efforttypes_predicate & has_global_perm("maka.add_efforttype")
add_perm("maka.add_efforttype_rule", add_efforttype_predicate)

edit_efforttype_predicate = view_efforttypes_predicate & has_global_perm("maka.change_efforttype")
add_perm("maka.edit_efforttype_rule", edit_efforttype_predicate)

delete_efforttype_predicate = view_efforttypes_predicate & has_global_perm("maka.delete_efforttype")
add_perm("maka.delete_efforttype_rule", delete_efforttype_predicate)

view_efforttypes_menu_predicate = has_person & (
    has_global_perm("maka.view_efforttype")
    | has_any_object("maka.view_efforttype", EffortType)
)
add_perm("maka.view_efforttypes_menu_rule", view_efforttypes_menu_predicate)

# View parent menu entry
view_menu_predicate = view_gradesets_menu_predicate | view_efforttypes_menu_predicate | view_efforts_menu_predicate
add_perm("maka.view_menu_rule", view_menu_predicate)
