import graphene

from aleksis.core.schema.base import FilterOrderList

from .effort import (
    EffortBatchCreateMutation,
    EffortBatchDeleteMutation,
    EffortBatchPatchMutation,
    EffortType,
    EffortTypeBatchCreateMutation,
    EffortTypeBatchDeleteMutation,
    EffortTypeBatchPatchMutation,
    EffortTypeType,
)
from .grade import (
    GradeBatchCreateMutation,
    GradeBatchDeleteMutation,
    GradeBatchPatchMutation,
    GradeType,
)
from .grade_set import (
    GradeChoiceBatchCreateMutation,
    GradeChoiceBatchDeleteMutation,
    GradeChoiceBatchPatchMutation,
    GradeChoiceType,
    GradeSetBatchCreateMutation,
    GradeSetBatchDeleteMutation,
    GradeSetBatchPatchMutation,
    GradeSetType,
)
from .shared_secret import SubmitSharedSecretMutation


class Query(graphene.ObjectType):
    grade_sets = FilterOrderList(GradeSetType)
    grade_choices = FilterOrderList(GradeChoiceType)
    grades = FilterOrderList(GradeType)
    effort_types = FilterOrderList(EffortTypeType)
    efforts = FilterOrderList(EffortType)

    shared_secret_status = graphene.Boolean()

    def resolve_shared_secret_status(root, info) -> bool:
        return info.context.session.get("maka_shared_secret_correct")


class Mutation(graphene.ObjectType):
    create_grade_sets = GradeSetBatchCreateMutation.Field()
    delete_grade_sets = GradeSetBatchDeleteMutation.Field()
    update_grade_sets = GradeSetBatchPatchMutation.Field()

    create_grade_choices = GradeChoiceBatchCreateMutation.Field()
    delete_grade_choices = GradeChoiceBatchDeleteMutation.Field()
    update_grade_choices = GradeChoiceBatchPatchMutation.Field()

    create_grades = GradeBatchCreateMutation.Field()
    delete_grades = GradeBatchDeleteMutation.Field()
    update_grades = GradeBatchPatchMutation.Field()

    create_effort_types = EffortTypeBatchCreateMutation.Field()
    delete_effort_types = EffortTypeBatchDeleteMutation.Field()
    update_effort_types = EffortTypeBatchPatchMutation.Field()

    create_efforts = EffortBatchCreateMutation.Field()
    delete_efforts = EffortBatchDeleteMutation.Field()
    update_efforts = EffortBatchPatchMutation.Field()

    submit_shared_secret = SubmitSharedSecretMutation.Field()
